@@ -9,12 +9,12 @@ use Exporter;
 our ( %EXPORT_TAGS, @ISA, @EXPORT_OK, @EXPORT, $VERSION );
 @ISA = qw( Exporter );
 
-$VERSION = '0.01.1';
+$VERSION = '0.01.2';
 
 our ($FILTER_REGEX, $NOT_FILTER_REGEX, $FILTER_ALL_REGEX, $MATCH_LOG_LEVEL_REGEX, $FILTER, $NOT_FILTER, $CNT );
 
 our ( $DUMPER, $log_level, $log );
-our ( $ll, $lll, $cll, $ell, $wll, $nll, $ill, $dll, $tll, $sll, $mll );
+our ( $ll, $all, $lll, $cll, $ell, $wll, $nll, $ill, $dll, $tll, $sll, $mll );
 
 # we don't normally want a stack trace on every log call
 our $STACK_TRACE = $ENV{STACK_TRACE} || 0;
@@ -30,13 +30,13 @@ if ( $ENV{LOG_USE_CARP} ) { # map { ($_ ? $_ : ()) } values %BIG_WARN_ON ) {
   $SIG{__WARN__} = \&cluck;
   $SIG{__DIE__}  = \&confess;
 }
-
-use constant LOG_LEVELS =>  qw( DEFAULT MESSAGE LOUD CLEAN QUIET
+# DEFAULT MESSAGE QUIET
+use constant LOG_LEVELS =>  qw( ALWAYS LOUD CLEAN
                                 EMERG ALERT CRIT ERROR WARN NOTICE INFO DEBUG
                                 TRACE SPEW );
 
 # if any $log->write(...)'s are in the calling code, and the log level is specified with one of the followin prepended with a '$'
-use constant DEFAULT_FILTER =>  qw( mll lll qll cll ell wll nll ill dll tll sll );
+use constant DEFAULT_FILTER =>  qw( all mll lll qll cll ell wll nll ill dll tll sll );
 
 
 %EXPORT_TAGS = (
@@ -52,8 +52,8 @@ use constant DEFAULT_FILTER =>  qw( mll lll qll cll ell wll nll ill dll tll sll
                 # this is useful for debugging when you may want a particular message to be displayed (simply delete the '$')
                 log_level_not_filtered => [                 DEFAULT_FILTER() ],
                 # these are utility methods for output formatting
-                 misc      => [ qw(space pad dump _caller) ],
-                       );
+                misc      => [ qw(space pad dump _caller) ],
+               );
 
 $EXPORT_TAGS{all} = [ map {@{$_}} values %EXPORT_TAGS ];
 $EXPORT_TAGS{initialize} = [ @{$EXPORT_TAGS{log_level}} ];
@@ -69,31 +69,35 @@ $EXPORT_TAGS{initialize} = [ @{$EXPORT_TAGS{log_level}} ];
 # while also allowing us to have more descriptive prefix (caller, etc...) when they are used for
 # development debugging
 
-( $ll, $lll, $cll, $ell, $wll, $nll, $ill, $dll, $tll, $sll, $mll)
-  = qw(DEFAULT LOUD CLEAN ERROR WARN NOTICE INFO DEBUG TRACE SPEW MESSAGE );
+( $ll, $all, $lll, $cll, $ell, $wll, $nll, $ill, $dll, $tll, $sll, $mll)
+  = qw(DEFAULT ALWAYS LOUD CLEAN ERROR WARN NOTICE INFO DEBUG TRACE SPEW MESSAGE );
 
 # these were(are?) actually apache constants for logging levels I think anything that gets in that
 # is preceded with a '_' gets [0] (numerical value) these return the uppercase(?) version of
 # themselves
 
-use constant LOG_CODE   => { MESSAGE => 0x000D,
+use constant LOG_CODE   => { ALWAYS  => 0x000E,
+                             STDERR  => 0x000E,
+                             STDOUT  => 0x000E,
+                             MESSAGE => 0x000D,
                              LOUD    => 0x000C,
-                                  CLEAN   => 0x000B,
-                                  EMERG   => 0x000A,
-                                  ALERT   => 0x0009,
-                                  CRIT    => 0x0008,
-                                  ERROR   => 0x0007,
-                                  WARN    => 0x0006,
-                                  NOTICE  => 0x0005,
-                                  INFO    => 0x0004,
-                                  DEBUG   => 0x0003,
-                                  TRACE   => 0x0002,
-                                  SPEW    => 0x0001,
-                                  QUIET   => 0x0000,
-                                  DEFAULT => 0x0003,# set equal to DEBUG
-                                };
+                             EMERG   => 0x000B,
+                             ALERT   => 0x000A,
+                             CRIT    => 0x0009,
+                             ERROR   => 0x0008,
+                             WARN    => 0x0007,
+                             CLEAN   => 0x0006,
+                             NOTICE  => 0x0005,
+                             INFO    => 0x0004,
+                             DEBUG   => 0x0003,
+                             TRACE   => 0x0002,
+                             SPEW    => 0x0001,
+                             QUIET   => 0x0000,
+                             DEFAULT => 0x0003,# set equal to DEBUG
+                           };
 
 use constant MESSAGE => 'MESSAGE'; # this will send an email to the appointed person
+use constant ALWAYS  => 'ALWAYS';
 use constant DEFAULT => 'DEFAULT';
 use constant LOUD    => 'LOUD';
 use constant CLEAN   => 'CLEAN';
@@ -123,6 +127,7 @@ our %APACHE_LEVELS = ( DEFAULT => INFO,
                        LOUD    => ERROR,
                        CLEAN   => ERROR,
                        QUIET   => DEBUG,
+                       ALWAYS  => ERROR,
                      );
 
 
@@ -131,6 +136,7 @@ our %APACHE_LEVELS = ( DEFAULT => INFO,
 # I have also made identically named scalars which if used will  cause the log messages to be filtered out
 # WARNING: without the `$' the log message WILL NOT be filtered out!
 use constant  ll  => DEFAULT;
+use constant  all => ALWAYS;
 use constant  lll => LOUD;
 use constant  cll => CLEAN;
 use constant  qll => QUIET;
@@ -254,7 +260,6 @@ sub log            { exists $_[1] ? $_[0]->{ log            } = $_[1] : $_[0]->{
 sub log_file       { exists $_[1] ? $_[0]->{ log_file       } = $_[1] : $_[0]->{ log_file      }; }
 sub log_level      { exists $_[1] ? $_[0]->{ log_level      } = $_[1] : $_[0]->{ log_level     }; }
 sub dump_refs      { exists $_[1] ? $_[0]->{ dump_refs      } = $_[1] : $_[0]->{ dump_refs     }; }
-sub skip_tmp       { exists $_[1] ? $_[0]->{ skip_tmp       } = $_[1] : $_[0]->{ skip_tmp      }; }
 sub handle_fatals  { exists $_[1] ? $_[0]->{ handle_fatals  } = $_[1] : $_[0]->{ handle_fatals }; }
 sub exclusive      { exists $_[1] ? $_[0]->{ exclusive      } = $_[1] : $_[0]->{ exclusive     }; }
 sub stack_trace    { exists $_[1] ? $_[0]->{ st             } = $_[1] : $_[0]->{ st            }; }
@@ -265,20 +270,18 @@ sub clone {
   my $self = shift;
   my $VAR1 = $self->dump( @_ );
   my $clone = eval $VAR1;
-  #  $log->write( LOUD, $log );
-  #  $log->write( CRIT, $clone );
-  $clone->{prefix} = $self->{prefix};
+  $clone->{prefix} = $self->{prefix} if ( UNIVERSAL::isa( $self, __PACKAGE__ ) and ref $self->{prefix} eq 'CODE' );
   return $clone;
 }
 
 our $default_handle_fatals = 1;
-our %init = ( log_file       => $ENV{LOG_EXCLUSIVE} || 'STDERR' ,
-                    log_level      => $log_level,
-                    dump_refs      => defined $ENV{LOG_DUMP_REFS} ? $ENV{LOG_DUMP_REFS} : 1 ,
-                    handle_fatals  => defined $ENV{LOG_HANDLE_FATALS} ? $ENV{LOG_HANDLE_FATALS} : $default_handle_fatals,
-                    exclusive      => $ENV{LOG_EXCLUSIVE} || '',
+our %init = ( log_file       => $ENV{LOG_FILE} || 'STDERR' ,
+              log_level      => $log_level,
+              dump_refs      => defined $ENV{LOG_DUMP_REFS} ? $ENV{LOG_DUMP_REFS} : 1 ,
+              handle_fatals  => defined $ENV{LOG_HANDLE_FATALS} ? $ENV{LOG_HANDLE_FATALS} : $default_handle_fatals,
+              exclusive      => $ENV{LOG_EXCLUSIVE} || '',
               prefix         => \&_default_prefix,
-                  );
+            );
 
 $log = __PACKAGE__->new();
 sub new {
@@ -436,10 +439,9 @@ sub _check_level {
       and $level=shift @$msg;
   
   my $log_level = $args->{log_level} ||= $self->{log_level}   || DEFAULT;
-  $level = $log_level if $log_level eq CLEAN; # ALWAYS output with a CLEAN log level
-#  print STDERR "$$ [@{[scalar localtime()]}]@{[_caller(0)]}-@{[uc$level]}-> \n";
-#  warn;
   my ( $_level, $_log_level ) = (@{LOG_CODE()}{$level, $log_level});
+  #  print STDERR "\nLEVELS: $log_level:$_log_level ... $level:$_level\n" if $self->{EVIL};
+
   if ( not defined $_level ) {
     unshift @$msg, ( $level = 'DEFAULT' );
     $log->write( ERROR, "Illegal log level '$level' setting it to $level");
@@ -448,7 +450,7 @@ sub _check_level {
     return undef;
   }
 
-  return ($level, $_level, $_log_level, $args);
+  return ($level, $_level, $log_level, $_log_level, $args);
 }
 
 sub write {
@@ -456,40 +458,45 @@ sub write {
   my $self  = shift;
   ref $self or $self = $log;
   my @msg = @_;
-  my ($level, $_level, $_log_level, $args)  = $self->_check_level( \@msg );
+  my ($level, $_level, $log_level, $_log_level, $args)  = $self->_check_level( \@msg );
   # this needs to be set up to log at any of severa levels which may be set simultaneously
   # eg log at WARN and TRACE
   # log levels should be a list
   # ie @_log_levels rather than $_log_level
   my $return = \@msg;
+  my $status = 1;
   if ( my $e = $self->{exclusive} ) {
-    $_level == $_log_level
-      or $e =~ /$level/
-        or return join( '', @$return );
+    $e =~ /$level/
+      or $status = 0;# or return join( '', @$return );
   } else {
-    $_level >= $_log_level or return join( '', map { defined $_ ? $_ : 'undef' } @$return );
+    $_level >= $_log_level 
+      or $status = 0;
+      #or return join( '', map { defined $_ ? $_ : 'undef' } @$return );
   }
-  #warn;
-  @msg = $self->_prepare_message( $level, $args, @msg );
-  $n = exists $args->{n} ? $args->{n} : ($self->{n} || "\n");
-  $return = $self->_actually_log( %$args, -level => $level, -message => $return );
-
-  if ( $level eq CRIT and $self->{handle_fatals} ) {
-    if ( my $email = $self->{email} ) {
-      # should we send a message to the bloke?
+  if ( $status ) {
+    #warn "STATUS: $status ::: $level:$_level ... $log_level:$_log_level";
+    @msg = $self->_prepare_message( $level, $args, @msg );
+    $n = exists $args->{n} ? $args->{n} : ($self->{n} || "\n");
+    $return = $self->_actually_log( %$args, -level => $level, -message => $return );
+    
+    if ( $level eq CRIT and $self->{handle_fatals} ) {
+      if ( my $email = $self->{email} ) {
+        # should we send a message to the bloke?
+      }
+      die "$0: `CRIT'ical error occurred :: $return";
     }
-    die "$0: `CRIT'ical error occurred :: $return";
-  }
-  if ( $level eq MESSAGE  ) {
-    if ( my $email = $args->{email} ? $args->{email} : $self->{email} ) {
-      # we should send a message to the bloke?
-    } else {
-      $log->write(ERROR, "No email address specified to send MESSAGE: $return");
+    if ( $level eq MESSAGE  ) {
+      if ( my $email = $args->{email} ? $args->{email} : $self->{email} ) {
+        # we should send a message to the bloke?
+      } else {
+        $log->write(ERROR, "No email address specified to send MESSAGE: $return");
+      }
     }
+    $BIG_WARN_ON{$level} and warn "$level : $return";
+    $n = undef;
   }
-  $BIG_WARN_ON{$level} and warn $return;
-  $n = undef;
-  return $return;
+  ref $return eq 'ARRAY' and $return = join('', @$return);
+  return wantarray ? ( $return, $status ) : $return ;
 }
 sub _actually_log {
   #print STDERR __PACKAGE__, " ", __LINE__, " ::: OH MY!\n";
@@ -625,9 +632,9 @@ FILEHANDLES : { # a cache of open filehandles for output
       
       $fh = $args->{fh} || $FHS{$file};
     }
-    #print STDOUT "FH: [$level] :: ", $fh, "\n";
+    #print STDOUT "FH: [$level] :: ", $fh, ":", fileno($fh), "\n";
     
-    return $fh if fileno($fh);
+    if ( $fh ) { return $fh if fileno($fh);}
     
     ( $fh = $FHS{$file} = new IO::File ) or die $!;
     
@@ -687,23 +694,167 @@ Log::Easy - Easy to use, feature rich general purpose logging utility
 =head1 SYNOPSIS
 
   use Log::Easy;
-  <BRIEF USAGE>
+  $log = new Log::Easy;
+
+=item $log->write({ OPTIONS }, <LOG_LEVEL>, @message );
+
+This is the main function for this package. If the first argument is a
+hash reference, it is taken as options to the logger for this log call
+ONLY, and may contain values for ANY of the options that the logger
+knows how to use. The first argument after the optional OPTIONS hash
+MUST BE the log level for this log. If it ends up being something that
+is not a log_level, then a default log_level is assigned to the write
+call. If the $log->write($log_level, @message) is equal to or greater
+than the $log->log_level() [or the $OPTIONS->{log_level}] then it will
+be output, otherwise it will not. 
+
+=item $log->clone( [$data] );
+
+make a duplicate of the supplied $data or this log object if no data
+is supplied. Uses data dumper to duplicate original, therefore CODE
+references are not necessarily handled, although the contents of
+prefix will be assigned to the new (cloned) object from the original.
+returns the cloned object
+
+=head2 OPTIONS
+
+=item $log->n();
+
+set the message terminator for this log object, each log message
+output will be terminated with the contents of this setting (default
+=> "\n")
+
+=item  $log->log();
+
+set the log object to use for actual write operation, this occurrs
+AFTER the decision as to whether or not this log message will be
+processed
+
+
+=item  $log->log_file();
+
+set the output file for messages processed through this log object (
+can be a file handle or file path, default => 'STDERR')
+
+
+=item  $log->log_level();
+
+set the thrshold for which messages will actiually be logged, only
+messages with a log_level set to a (nuerically) higher value will be
+output (default => WARN)
+
+
+=item  $log->dump_refs();
+
+set the behavior when references are encountered in the message list
+contents (default => 1) if true, use data dumper to dump out the
+references
+
+
+=item  $log->handle_fatals();
+
+if true, log module will terminate program execution on any log calls
+marked as fatal (CRIT for now [maybe add EMERG], default => 1)
+
+
+=item  $log->exclusive();
+
+if true, and set to a colon delimited set of log_level indicators,
+this log object will only output its message if the current log call
+is for a level listed in exclusive (default => undef)
+
+
+=item  $log->stack_trace();
+
+if true (and this log message is of sufficient log_level to be
+output), issue a pretty little stack trace for the log call
+
+
+=item  $log->email(); #NOT YET IMPLEMENTED
+
+for a log level of MESSAGE, send the log message to the email address
+listed here, if not set, and a MESSAGE log comes in, then send the
+email to the owner of the process
+
+
+=item  $log->prefix();
+
+set ther prefix that each log message will have, may be either a
+string, or a CODE ref
+
 
 =head1 DESCRIPTION
 
-<LONG DESC>
+  This logging facility has many features developers may find handy.
+
+=item FILTERING
+
+One of the coolest things is FILTERING (see log_level_filtered). Using
+filtering we can greatly decrease the performance penalty of copious
+log entries (each requireing >1 subroutine call) by filtering out log
+messages at compile time. Here is the general idea:
+
+ # FILTERED
+ $log->write($dll, "Nifty stuff");
+
+ # NOT-FILTERED, but will issue a warning if $ENV{WARN_FILTER} is true.
+ $log->write(dll, "Nifty stuff");
+
+ # NOT-FILTERED, functionally equivalent to previous example
+ $log->write(DEBUG, "Nifty stuff");
 
 
 =head2 EXPORT
 
 None by default.
 
+=over 8
+
+=item log
+
+exports a log object into your namespace. This is most likely an
+application global log object meaning that if you wan't specific
+logging behavior in any given package, you'll probably want to
+intanntiate a new log object
+
+=item log_level
+
+exports several constants into your namespace:
+   DEFAULT MESSAGE LOUD CLEAN QUIET
+   EMERG ALERT CRIT ERROR WARN NOTICE INFO DEBUG
+   TRACE SPEW
+
+
+=item log_level_filtered
+
+these are scalar variable ($) aliases for the constants mentioned in
+log_level, the convention is that each alias begins with the same
+letter as the real log_level, followed by 2 `l's (eg ERROR => $ell,
+DEBUG => $dll, etc)
+
+These log_level specifiers, when used with the leading dollar sign MAY
+BE FILTERED OUT depending upon the settings for $ENV{FILTER}
+
+=item log_level_not_filtered
+
+in name, these are identical to the log_level_filtered variables, but
+they are implemented as constants (no leading $) and they ARE NOT
+SUBJECT TO FILTERING AS IS THE CASE FOR log_level_filtered
+
+=back
 
 =head1 HISTORY
 
 =over 8
 
 =item 0.01
+Basic package w/ full functionality, but no docs yet
+
+=item 0.01.1
+Basic package w/ full functionality, but no docs yet, used make dist to create pause upload
+
+=item 0.01.2
+Added some documentation
 
 =back
 
@@ -712,7 +863,17 @@ None by default.
 =over 8
 
 =item 0.01
+
 DOCUMENTATION DOCUMENTATION DOCUMENTATION DOCUMENTATION DOCUMENTATION !!!!!
+
+=item 0.02
+
+change the codes which represent the log_levels such that $xll style
+log calls can have slightly different behavior in terms of being
+programmer-debug calls, which will probably want to have detailed
+prefix informtaion, while non-programmer-debug calls may want
+different prefix info
+
 =back
 
 
